import Foundation
import CoreLocation

public class BackgroundBeaconScanner: NSObject, CLLocationManagerDelegate{
    
    public static let shared = BackgroundBeaconScanner();
    
    private let locationManager = CLLocationManager()
    private var customData: Dictionary<String,Any>?
    
    open var delegate: SnowMBackgroundScanningServiceDelegate?
    

    override init(){
        super.init()
        locationManager.pausesLocationUpdatesAutomatically = false
        if #available(iOS 9.0, *) {
            locationManager.allowsBackgroundLocationUpdates = true
        }
        locationManager.delegate = self;
    }
    

    
    public func scanBeacons(uuids:[String], customData: Dictionary<String,Any>){
        locationManager.requestAlwaysAuthorization()
        locationManager.startUpdatingLocation()
        self.customData = customData
        uuids.forEach { (id) in
            let uuid = UUID(uuidString: id)
            let region = CLBeaconRegion(proximityUUID: uuid!, identifier: id)
            locationManager.startMonitoring(for: region)
            locationManager.startRangingBeacons(in: region)
        }
    }
    
    public func locationManager(_ manager: CLLocationManager, didUpdateLocations locations: [CLLocation]) {
        delegate?.onLocationUpdated(locations: locations)
    }
    
    
    public func removeScanner(){
        locationManager.monitoredRegions.forEach { (region) in
            locationManager.stopMonitoring(for: region)
        }
    }
    
    public func locationManager(_ manager: CLLocationManager, didRangeBeacons beacons: [CLBeacon], in region: CLBeaconRegion) {
        let snowmBeacons:[SnowMBeacon] = beacons.map {
            let snowmBeacon = SnowMBeacon()
            snowmBeacon.uuid = $0.proximityUUID.uuidString
            snowmBeacon.major = $0.major.intValue
            snowmBeacon.minor = $0.minor.intValue
            snowmBeacon.txPower = 0
            snowmBeacon.macAddress = "Unavailable"
            snowmBeacon.distance = $0.accuracy.distance(to: 1)
            snowmBeacon.rssi = $0.rssi
            return snowmBeacon
        }
        if delegate != nil{
            delegate?.onBeaconDetected(beacons: snowmBeacons, customData: customData)
        }
    }
    
    public func locationManager(_ manager: CLLocationManager, rangingBeaconsDidFailFor region: CLBeaconRegion, withError error: Error) {
    }
}
